package edu.kit.ipd.descartes.adaptation.util;

import edu.kit.ipd.descartes.adaptation.TacticExecutor;
import edu.kit.ipd.descartes.mm.adaptation.WeightedTactic;

public class BlueYonderExperimentTracker implements IObserver {

    private static final String RESULT_NAME = "Xreq-par-$1Yps-desc4-gw-desc3-db-desc2-predict-Z ps=$2-req=5-size=50000";
    private static BlueYonderExperimentTracker instance = null;
    private int psOnDescOne = 0;
    private int psOnDescFour = 1;
    private int psOnDescOneBackup;
    private int psOnDescFourBackup;

    public void setSubject(TacticExecutor te) {
        te.register(this);
    }

    public String getCurrentExperimentName() {
        String result = RESULT_NAME;

        if (psOnDescOne > 0)
            result = RESULT_NAME.replace("$1", psOnDescOne + "ps-desc1-");
        else
            result = RESULT_NAME.replace("$1", "");

        result = result.replace("$2", Integer.toString(psOnDescFour));
        return result;
    }

    public void track(WeightedTactic tactic) {
        String execTacticName = tactic.getName();
        backupValues();

        if (execTacticName.contains("Consolidate")) {
            int loopCount = Integer.parseInt(tactic.getUsedTactic().getInputParams().get(0).getValue());
            if (execTacticName.contains("Low")) {
                psOnDescFour = psOnDescFour - loopCount;
                psOnDescOne = psOnDescOne + loopCount;
            } else if (execTacticName.contains("High")) {
                psOnDescOne = psOnDescOne - loopCount;
                psOnDescFour = psOnDescFour + loopCount;
            }
        } else if (execTacticName.contains("Low")) {
            psOnDescOne++;
            assert psOnDescFour > 0;
            if (execTacticName.contains("Migrate"))
                psOnDescFour--;
        } else if (execTacticName.contains("High")) {
            psOnDescFour++;
            assert psOnDescOne > 0;
            if (execTacticName.contains("Migrate"))
                psOnDescOne--;
        }
    }

    private void backupValues() {
        psOnDescOneBackup = psOnDescOne;
        psOnDescFourBackup = psOnDescFour;
    }

    @Override
    public void undo() {
        resetValues();
    }

    private void resetValues() {
        psOnDescOne = psOnDescOneBackup;
        psOnDescFour = psOnDescFourBackup;
    }

    public static BlueYonderExperimentTracker getInstance() {
        if (instance == null)
            instance = new BlueYonderExperimentTracker();
        return instance;
    }
}
